use libc::{c_char, c_int};
use std::ffi::CString;
use std::os::unix::ffi::OsStrExt;
use std::path::Path;
use std::io::{Error, Result};

#[link(name = "hash")]
extern {
    fn hash_md5_file(path: *const c_char,
                     flags: c_int,
                     hash: &mut [u8;16]) -> c_int;
}

pub fn md5_file(path: &Path,
            flags: c_int) -> Result<String>
{
    let mut hash: [u8;16] = [0;16];
    let cstring = CString::new(path.as_os_str().as_bytes()).unwrap();
    if unsafe { hash_md5_file(cstring.as_ptr(), flags, &mut hash) } < 0 {
        Err(Error::last_os_error())
    } else {
        let s: Vec<String> = hash.iter().map(
            |b| format!("{:02x}", b)).collect();
        Ok(s.join(""))
    }
}

#[link(name = "hash")]
extern {
    fn hash_md5_symlink(path: *const c_char,
                     flags: c_int,
                     hash: &mut [u8;16]) -> c_int;
}

pub fn md5_symlink(path: &Path,
                   flags: c_int) -> Result<String>
{
    let mut hash: [u8;16] = [0;16];
    let cstring = CString::new(path.as_os_str().as_bytes()).unwrap();
    if unsafe { hash_md5_symlink(cstring.as_ptr(), flags, &mut hash) } < 0 {
        Err(Error::last_os_error())
    } else {
        let s: Vec<String> = hash.iter().map(
            |b| format!("{:02x}", b)).collect();
        Ok(s.join(""))
    }
}

#[link(name = "hash")]
extern {
    fn hash_sha512_file(path: *const c_char,
                        flags: c_int,
                        hash: &mut [u8;64]) -> c_int;
}

pub fn sha512_file(path: &Path,
               flags: c_int) -> Result<String>
{
    let mut hash: [u8;64] = [0;64];
    let cstring = CString::new(path.as_os_str().as_bytes()).unwrap();
    if unsafe { hash_sha512_file(cstring.as_ptr(), flags, &mut hash) } < 0 {
        Err(Error::last_os_error())
    } else {
        let s: Vec<String> = hash.iter().map(
            |b| format!("{:02x}", b)).collect();
        Ok(s.join(""))
    }
}

#[link(name = "hash")]
extern {
    fn hash_sha512_symlink(path: *const c_char,
                           flags: c_int,
                           hash: &mut [u8;64]) -> c_int;
}

pub fn sha512_symlink(path: &Path,
                      flags: c_int) -> Result<String>
{
    let mut hash: [u8;64] = [0;64];
    let cstring = CString::new(path.as_os_str().as_bytes()).unwrap();
    if unsafe { hash_sha512_symlink(cstring.as_ptr(), flags, &mut hash) } < 0 {
        Err(Error::last_os_error())
    } else {
        let s: Vec<String> = hash.iter().map(
            |b| format!("{:02x}", b)).collect();
        Ok(s.join(""))
    }
}

