#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include <openssl/sha.h>
#include <errno.h>
#include <limits.h>

#define BLOCKSIZE 4096

int hash_sha512_symlink(char *filename,
                        int flags,
                        unsigned char *hash)
{
  char buf[PATH_MAX];
  int n;
  
  n = readlink(filename, buf, sizeof(buf));
  if (n < 0) {
    return -1;
  } else if (n == PATH_MAX) {
    errno = ENAMETOOLONG;
    return -1;
  } else {
    SHA512_CTX ctx;

    SHA512_Init(&ctx);
    SHA512_Update(&ctx, buf, n);
    SHA512_Final(hash, &ctx);
  }
  return 0;
}

int hash_sha512_file(char *filename,
                        int flags,
                        unsigned char *hash)
{
  int fd;
  
  fd = open (filename, O_RDONLY);
  if (fd < 0) {
    return -1;
  } else {
    SHA512_CTX ctx;
    unsigned char buffer[BLOCKSIZE];
    int count;
    time_t now;
      
    time(&now);
    SHA512_Init(&ctx);
    while ((count = read(fd, buffer, BLOCKSIZE)) > 0) {
      SHA512_Update(&ctx, buffer, count);
    }
    SHA512_Final(hash, &ctx);

    goto out_close;
  }
out_close:
  close(fd);
  return 0;
}
